import pandas as pd
import pyarrow.parquet as pq
import glob
import numpy as np
import os
import io

from matplotlib import pyplot
from pandas.plotting import lag_plot
from pandas.plotting import autocorrelation_plot
from pandas import read_csv
from pandas.plotting import autocorrelation_plot
from statsmodels.tsa.ar_model import AutoReg
from statsmodels.tsa.stattools import adfuller
from statsmodels.graphics.tsaplots import plot_pacf
import pmdarima as pm
from math import sqrt

from numpy import mean
from numpy import std
from numpy import absolute
from numpy import arange

from scipy import stats
import pmdarima as pmd
import matplotlib.pyplot as plt
import pybats
from pybats.loss_functions import MAPE
from pybats.analysis import analysis
from pybats.point_forecast import median

from sklearn.model_selection import cross_val_score
from sklearn.metrics import accuracy_score
from sklearn.model_selection import RepeatedKFold
from sklearn.model_selection import GridSearchCV
from sklearn.linear_model import Lasso
from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

desired_width = 320
pd.set_option('display.width', desired_width)
pd.set_option('display.max_columns', 10)
pd.options.mode.chained_assignment = None

#Loading the datasets
#Loading all unemployment files
filesUnemployment = glob.glob('unemployment_data_us/adjusted/*')
filesUnemployment.sort()

filesUnemploymentCountry = glob.glob('unemployment_usa.csv')
filesUnemploymentCountry.sort()

#Loading all US states files
files = glob.glob('total_grouped_job_numbers.parquet')
# files = glob.glob('../data_bg_us_v1/us/*/*.parquet')
files.sort()

#Loading wages data
filesWages = glob.glob('total_grouped_mean_salaries.parquet')
filesWages.sort()
fileWages = filesWages[0]

#Loading wages data for the country
filesWagesUSA = glob.glob('total_grouped_mean_salaries_country.parquet')
filesWagesUSA.sort()
fileWagesUSA = filesWagesUSA[0]

df = pd.read_parquet(fileWages)
print(df)

df = pd.read_parquet(fileWagesUSA)
print(df)

filesUSA = glob.glob('total_grouped_jobs_country.parquet')
filesUSA.sort()
fileUSA=filesUSA[0]

def bayes_forecast(iv, dv):
    '''
    This functions runs the Pybats algorithm by taking two parameters: an independent variable matrix and a dependent variable.
    Both elements must be sequential time series.
    '''
    # first check if the iv = None, indicating this would be a univariate series
    if iv is None:
        x = None
    else:
        x = iv.values

    y = dv.values[:100]
    yTest = dv.values[100:]

    # set the one-step-ahead value; by default we want 1
    k = 1
    forecast_start = 100
    forecast_end = len(y) - 1

    mod, samples = analysis(Y=y, X=x[:100], family='normal',
                            forecast_start=forecast_start,
                            forecast_end=forecast_end,
                            k=k,
                            ntrend=1,
                            prior_length=3,
                            deltrend=0.94,
                            delregn=0.90,
                            delVar=0.98,
                            delSeas=0.98,
                            rho=6, #ideal 6, higher disproportion is 0.3
                            )
    forecast = median(samples)

    mae = mean_absolute_error(yTest, forecast)
    print("MAE:",format(mae, ".3f"))
    mse = mean_squared_error(yTest, forecast)
    print("MSE: ", format(mse, ".3f"))
    r2 = r2_score(yTest, forecast)
    print("R2: ", format(r2, ".3f"))
    return mod, forecast, samples, y, mae, mse, r2

#Generating results
resultsDf = pd.DataFrame()
for unFile in filesUnemployment[:]:
    tempArray = []
    csvSample = pd.read_csv(unFile, skiprows=0, usecols = [i for i in range(2)])
    csvSample.columns=['date', 'unrate']

    #Getting the name of the state
    fileName_absolute = os.path.basename(unFile)
    tempArray.append(fileName_absolute[:2])

    #Dropping last month (May 2021) as it is not complete
    csvSample.drop(csvSample.tail(1).index, inplace=True)

    # Changing type to date
    csvSample.loc[:, 'date'] = pd.to_datetime(csvSample.loc[:, 'date'], format='%Y-%m-%d')

    #Changing type to float
    csvSample.loc[:, 'unrate'] = csvSample.loc[:, 'unrate'].astype(float)

    #Accessing data for a particular state
    jobsGrupedDf = pd.read_parquet(files[0])
    selectedColsDf = jobsGrupedDf.loc[jobsGrupedDf['CanonState'] == fileName_absolute[:2]]
    selectedColsDf.loc[:, 'JobDate'] = pd.to_datetime(selectedColsDf.loc[:,'JobDate'])

    # Accessing wage data
    jobsGrupedDf = pd.read_parquet(fileWages)
    wagesDf = jobsGrupedDf.loc[jobsGrupedDf['CanonState'] == fileName_absolute[:2]]
    selectedColsDf['MeanSalary'] = wagesDf['MeanSalary']

    #Merging relevant column
    selectedColsDf.loc[:, 'Unrate'] = csvSample.loc[:,'unrate'].values
    selectedColsDf.loc[:, 'Unrate'] = selectedColsDf.loc[:,'Unrate'].astype(float)
    print(' ')
    print('STATE', fileName_absolute[:2])

    #BAYESIAN
    selectedColsDf.set_index('JobDate')

    #Defining X and Y sets
    newX = selectedColsDf.iloc[:, 2:4]
    newY = selectedColsDf.Unrate

    print("BAYESIAN MULTIVARIATE")
    mv_mod, mv_for, mv_samp, mv_y, bmae, bmse, br2 = bayes_forecast(newX, newY)
    tempArray.append(format(bmae, ".3f"))
    tempArray.append(format(bmse, ".3f"))
    tempArray.append(format(br2, ".3f"))

    # AUTO REGRESSION
    # Differencing data
    csvSample['unrate']=csvSample['unrate'].diff(1)

    #Dropping first month (Jan 2020) because it is NaN due to differencing
    csvSample.drop(csvSample.head(1).index, inplace=True)
    csvSample.index = pd.DatetimeIndex(csvSample.index).to_period('M')

    train = csvSample['unrate'][:100]
    test = csvSample['unrate'][100:136]

    ar_model = AutoReg(endog=train, lags=5).fit()
    # Make the predictions
    pred = ar_model.predict(start=len(train), end=(len(csvSample)-1), dynamic=False)
    armae = mean_absolute_error(test, pred)
    armse = mean_squared_error(test, pred)
    arr2 = r2_score(test, pred)
    print('AUTO REGRESSION')
    print('MAE', format(armae, ".3f"))
    print("MSE: ", format(armse, ".3f"))
    print("R2: ", format(arr2, ".3f"))
    tempArray.append(format(armae, ".3f"))
    tempArray.append(format(armse, ".3f"))
    tempArray.append(format(arr2, ".3f"))
    newSeries = pd.Series(tempArray)
    resultsDf = resultsDf.append(newSeries, ignore_index = True)

print(resultsDf)

#COUNTRY COUNT
print('')
print("COUNTRY")
csvSample = pd.read_csv(filesUnemploymentCountry[0], skiprows=0, usecols=[i for i in range(2)])
csvSample.columns = ['date', 'unrate']

# Dropping last month (May 2021) as it is not complete
csvSample.drop(csvSample.tail(1).index, inplace=True)

# Changing type to date
csvSample.loc[:, 'date'] = pd.to_datetime(csvSample.loc[:, 'date'], format='%Y-%m-%d')

usaDf = pd.read_parquet(fileUSA)
usaDf.loc[:, 'JobDate'] = pd.to_datetime(usaDf.loc[:, 'JobDate'])

# Accessing wage data
jobsGroupedDf = pd.read_parquet(fileWagesUSA)
usaDf['MeanSalary'] = jobsGroupedDf['MeanSalary']

# Merging relevant column
usaDf.loc[:, 'Unrate'] = csvSample.loc[:, 'unrate'].values
usaDf.loc[:, 'Unrate'] = usaDf.loc[:, 'Unrate'].astype(float)

#Redefining variables
newX = usaDf.iloc[:,1:3]
newY = usaDf.Unrate

print("BAYESIAN MULTIVARIATE")
mv_mod, mv_for, mv_samp, mv_y, bmae, bmse, br2 = bayes_forecast(newX, newY)

# Differencing data
csvSample['unrate']=csvSample['unrate'].diff(1)
csvSample.drop(csvSample.head(1).index, inplace=True)
csvSample.index = pd.DatetimeIndex(csvSample.index).to_period('M')
train = csvSample['unrate'][:100]
test = csvSample['unrate'][100:136]

#AUTO REGRESSION
ar_model = AutoReg(endog=train, lags=5).fit()
# Make the predictions
pred = ar_model.predict(start=len(train), end=(len(csvSample)-1), dynamic=False)
armae = mean_absolute_error(test, pred)
armse = mean_squared_error(test, pred)
arr2 = r2_score(test, pred)
print('AUTO REGRESSION')
print('MAE', format(armae, ".3f"))
print("MSE: ", format(armse, ".3f"))
print("R2: ", format(arr2, ".3f"))

tempArray = []
tempArray.append('USA')
tempArray.append(format(bmae, ".3f"))
tempArray.append(format(bmse, ".3f"))
tempArray.append(format(br2, ".3f"))
tempArray.append(format(armae, ".3f"))
tempArray.append(format(armse, ".3f"))
tempArray.append(format(arr2, ".3f"))
newSeries = pd.Series(tempArray)
resultsDf = resultsDf.append(newSeries, ignore_index = True)

resultsDf.columns=['State', 'BDLM MAE', 'BDLM MSE', 'BDLM R2', 'AR MAE', 'AR MSE', 'AR R2']
print(resultsDf)